From c967493799fe261c6bc0d3d36a5664763ec29f31 Mon Sep 17 00:00:00 2001
From: Takashi Yano <takashi.yano@nifty.ne.jp>
Date: Thu, 16 Sep 2021 21:27:24 +0900
Subject: [PATCH 2/3] Cygwin: pipe: Fix error handling in
 fhandler_pip::create().

- Currently, error handling in fhandler_pipe::create() is broken.
  This patch fixes that.
---
 winsup/cygwin/fhandler_pipe.cc | 43 ++++++++++++++++++++++++----------
 1 file changed, 31 insertions(+), 12 deletions(-)

diff --git a/winsup/cygwin/fhandler_pipe.cc b/winsup/cygwin/fhandler_pipe.cc
index c2f4353f6..fd2ab87af 100644
--- a/winsup/cygwin/fhandler_pipe.cc
+++ b/winsup/cygwin/fhandler_pipe.cc
@@ -831,32 +831,51 @@ fhandler_pipe::create (fhandler_pipe *fhs[2], unsigned psize, int mode)
 	  NtClose (r);
 	  delete fhs[1];
 	  NtClose (w);
+	  goto out;
 	}
-      else
+      fhs[0]->set_read_mutex (mtx);
+
+      fhs[0]->select_sem = CreateSemaphore (sa, 0, INT32_MAX, NULL);
+      if (!fhs[0]->select_sem)
 	{
-	  fhs[0]->set_read_mutex (mtx);
-	  res = 0;
+	  CloseHandle (fhs[0]->read_mtx);
+	  delete fhs[0];
+	  NtClose (r);
+	  delete fhs[1];
+	  NtClose (w);
+	  goto out;
 	}
-      fhs[0]->select_sem = CreateSemaphore (sa, 0, INT32_MAX, NULL);
-      if (fhs[0]->select_sem)
-	DuplicateHandle (GetCurrentProcess (), fhs[0]->select_sem,
-			 GetCurrentProcess (), &fhs[1]->select_sem,
-			 0, sa->bInheritHandle, DUPLICATE_SAME_ACCESS);
+      if (!DuplicateHandle (GetCurrentProcess (), fhs[0]->select_sem,
+			    GetCurrentProcess (), &fhs[1]->select_sem,
+			    0, sa->bInheritHandle, DUPLICATE_SAME_ACCESS))
+	{
+	  CloseHandle (fhs[0]->read_mtx);
+	  CloseHandle (fhs[0]->select_sem);
+	  delete fhs[0];
+	  NtClose (r);
+	  delete fhs[1];
+	  NtClose (w);
+	  goto out;
+	}
+
       if (!DuplicateHandle (GetCurrentProcess (), r,
 			    GetCurrentProcess (), &fhs[1]->query_hdl,
 			    FILE_READ_DATA, sa->bInheritHandle, 0))
 	{
+	  CloseHandle (fhs[0]->read_mtx);
 	  CloseHandle (fhs[0]->select_sem);
 	  delete fhs[0];
-	  CloseHandle (r);
+	  NtClose (r);
 	  CloseHandle (fhs[1]->select_sem);
 	  delete fhs[1];
-	  CloseHandle (w);
+	  NtClose (w);
+	  goto out;
 	}
-      else
-	res = 0;
+
+      res = 0;
     }
 
+out:
   debug_printf ("%R = pipe([%p, %p], %d, %y)", res, fhs[0], fhs[1], psize, mode);
   return res;
 }
-- 
2.33.0

